@@ -24,12 +24,15 @@ package GroupsConfig;
 
 use base qw(StbObject);
 use Net::DBus::Exporter ($Utils::Backend::DBUS_PREFIX);
+use GroupConfig;
 use Users::Groups;
 use Users::Users;
 
-my $OBJECT_NAME = "GroupsConfig";
+my $OBJECT_NAME = "GroupsConfig2";
 my $OBJECT_PATH = "$Utils::Backend::DBUS_PATH/$OBJECT_NAME";
-my $format = [[ "array", [ "struct", "string", "string", "int32", [ "array", "string" ]]], "int32", "int32" ];
+
+# array of groups, min gid, max gid
+my $format = [ [ "array", $GroupConfig::GROUP_FORMAT ], "uint32", "uint32" ];
 
 sub new
 {
