@@ -87,7 +87,7 @@ use strict;
 use warnings;
 
 BEGIN {
-    our $VERSION = '0.33.4';
+    our $VERSION = '0.33.5';
     require XSLoader;
     XSLoader::load('Net::DBus', $VERSION);
 }
@@ -168,6 +168,11 @@ attached to the main L<Net::DBus::Reactor> event loop.
 
 sub system {
     my $class = shift;
+    my %params = @_;
+    if ($params{private}) {
+	return $class->_new(Net::DBus::Binding::Bus->new(type => &Net::DBus::Binding::Bus::SYSTEM, private => 1), @_);
+    }
+
     unless ($bus_system) {
 	$bus_system = $class->_new(Net::DBus::Binding::Bus->new(type => &Net::DBus::Binding::Bus::SYSTEM), @_);
     }
@@ -186,6 +191,11 @@ attached to the main L<Net::DBus::Reactor> event loop.
 
 sub session {
     my $class = shift;
+    my %params = @_;
+    if ($params{private}) {
+	return $class->_new(Net::DBus::Binding::Bus->new(type => &Net::DBus::Binding::Bus::SESSION, private => 1), @_);
+    }
+
     unless ($bus_session) {
 	$bus_session = $class->_new(Net::DBus::Binding::Bus->new(type => &Net::DBus::Binding::Bus::SESSION), @_);
     }
@@ -193,22 +203,6 @@ sub session {
 }
 
 
-=item my $bus = Net::DBus->test(%params);
-
-Returns a handle for a virtual bus for use in unit tests. This bus does
-not make any network connections, but rather has an in-memory message
-pipeline. Consult L<Net::DBus::Test::MockConnection> for further details
-of how to use this special bus.
-
-=cut
-
-# NB. explicitly do *NOT* cache, since unit tests
-# should always have pristine state
-sub test {
-    my $class = shift;
-    return $class->_new(Net::DBus::Test::MockConnection->new());
-}
-
 =item my $bus = Net::DBus->new($address, %params);
 
 Return a connection to a specific message bus.  The C<$address>
@@ -243,7 +237,7 @@ sub _new {
 
     unless ($params{nomainloop}) {
 	if (exists $INC{'Net/DBus/Reactor.pm'}) {
-	    my $reactor = Net::DBus::Reactor->main;
+	    my $reactor = $params{reactor} ? $params{reactor} : Net::DBus::Reactor->main;
 	    $reactor->manage($self->get_connection);
 	}
 	# ... Add support for GLib and POE
