@@ -89,4 +89,13 @@ sub monitor_files
   }
 }
 
+sub init_file_monitor
+{
+  return if (!$has_gamin);
+
+  # should not use internal stuff in $fm like that
+  Net::DBus::Reactor->main->add_read ($fm->{conn}->fd (),
+                                      Net::DBus::Callback->new(method => \&Utils::Monitor::do_monitor_files));
+}
+
 1;
